package jp.co.rakus;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Hashtable;
import java.util.Map;
import java.security.MessageDigest;

import javax.net.ssl.HttpsURLConnection;

import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * APIのサンプル実装
 */
public class SampleImportApi {

    private static final String REQUEST_CONTENT_TYPE = "multipart/form-data;boundary=";

    private static final String CRLF = "\r\n";

    private static final String BOUNDARY = "-+-+-+-+-+-+-+-+-Boundary_";

    // 接続先URLに利用する設定
    // https://●●●●●/■■■■■/api/v1/custcsv/import
    private final static String DOMAIN = "●●●●●";
    private final static String ACCOUNT = "■■■■■";
    private final static String API_URL = String.format("https://%s/%s/api/v1/custcsv/import", DOMAIN, ACCOUNT);

    // APIトークン
    // 各種設定 > 各種設定メニュー > 基本設定 > APIトークン で発行
    private final static String API_TOKEN = "<<APIトークン>>";

    /**
     * メインメソッド
     *
     * @param args コマンドライン引数（未使用）
     * @throws IOException IOエラーが発生した場合
     */
    public static void main(final String[] args) throws IOException {
        try {
            // jsonデータ作成
            final Map<String, Object> requestJson = new Hashtable<>();
            requestJson.put("import_type", "★");             // 0:登録, 1:削除, 2:停止, 3:禁止, 5:可能
            requestJson.put("is_overwrite", "★★");          // 1:上書きする, 0:上書きしない
            requestJson.put("overwrite_type", "★");          // 1:元の値を残す, 2:空白で上書きする
            requestJson.put("gid", "★");                     // 配信グループID（配信グループへの処理であれば必須です。）
            // requestJson.put("spid", "★");                 // ステップメールプランID（ステップメールプランへの処理であれば必須です。）
            requestJson.put("is_synchronize", "★★");        // 1:配信停止にする, 0:変更しない
            requestJson.put("report_option", "★");           // 0:送信しない, 1:送信する, 2:エラー時のみ送信する
            requestJson.put("login_id", "★★");              // ユーザID

            // 顧客リストCSV
            final String custCsvPath = "★★★★★";

            // API実行
            final Map<String, Object> responseJson = importCustCsv(requestJson, custCsvPath);

            if (Integer.parseInt(responseJson.get("code").toString()) == HttpURLConnection.HTTP_OK) {
                System.out.println("処理に成功しました");
                for (final String key : responseJson.keySet()) {
                    System.out.println("  " + key + " : " + responseJson.get(key));
                }
                System.exit(0);
            } else {
                System.err.println("処理に失敗しました");
                for (final String key : responseJson.keySet()) {
                    System.err.println("  " + key + " : " + responseJson.get(key));
                }
                System.exit(1);
            }
        } catch (final Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * 顧客データ取込を行う
     *
     * @param リクエストJSON
     * @param 顧客データ(CSV)のパス
     *
     * @return レスポンスJSON
     * @throws IOException IOエラーが発生した場合
     */
    private static Map<String, Object> importCustCsv(final Map<String, Object> requestJson, final String filepath) throws IOException {

        final URL url = new URL(API_URL);
        final HttpsURLConnection https = (HttpsURLConnection) url.openConnection();
        https.setRequestMethod("POST");
        https.setRequestProperty("X-HM-apitoken", API_TOKEN);
        final String newBoundary = BOUNDARY + getUniqueString();

        https.setRequestProperty("Content-Type", REQUEST_CONTENT_TYPE + newBoundary);
        https.setDoOutput(true);

        // リクエストボディを作成
        final StringBuilder importRequestBody = new StringBuilder();

        // jsonを追加
        importRequestBody.append("--" + newBoundary).append(CRLF);
        importRequestBody.append("Content-type: application/json; charset=utf-8").append(CRLF);
        importRequestBody.append("Content-Disposition: form-data; name=\"import_settings\";").append(CRLF);
        importRequestBody.append(CRLF);
        importRequestBody.append(new ObjectMapper().writeValueAsString(requestJson)).append(CRLF);

        importRequestBody.append("--" + newBoundary).append(CRLF);
        importRequestBody.append("Content-Type: text/csv").append(CRLF);
        importRequestBody
                .append("Content-Disposition: form-data; name=\"customer_list\"; filename=\"importDataSample.csv\";")
                .append(CRLF);
        importRequestBody.append(CRLF);

        // POST送信
        final BufferedOutputStream bos = new BufferedOutputStream(https.getOutputStream());
        bos.write(importRequestBody.toString().getBytes(StandardCharsets.UTF_8));
        bos.write(getCsv(filepath).getBytes(StandardCharsets.UTF_8));
        bos.write(new String("--" + newBoundary + "--").getBytes(StandardCharsets.UTF_8));
        bos.flush();
        bos.close();

        // レスポンス取得
        InputStreamReader streamReader = null;
        try {
            // 正常系
            streamReader = new InputStreamReader(https.getInputStream());
        } catch (final Exception e) {
            // 異常系
            streamReader = new InputStreamReader(https.getErrorStream());
        }

        final BufferedReader response = new BufferedReader(streamReader);
        String line = null;
        final StringBuilder responseBody = new StringBuilder();
        while ((line = response.readLine()) != null) {
            responseBody.append(line).append(CRLF);
        }

        return parseJson(responseBody.toString());
    }

    /**
     * csvデータを取得する。
     *
     * @param filepath 顧客データ(CSV)のファイルパス
     * @return csvデータの文字列
     */
    private static String getCsv(final String filepath) throws IOException {
        final BufferedReader br = new BufferedReader(new FileReader(filepath));
        final StringBuilder sb = new StringBuilder();

        String line;
        while ((line = br.readLine()) != null) {
            sb.append(line).append(CRLF);
        }
        br.close();

        return sb.toString();
    }

    /**
     * JSONレスポンスを解析する。
     *
     * @param responseBody レスポンス文字列
     * @return レスポンス文字列をJSONとしてパースしたMapオブジェクト
     * @throws IOException IOエラーが発生した場合
     */
    @SuppressWarnings("unchecked")
    private static Map<String, Object> parseJson(final String responseBody) throws IOException {
        // JacksonやJSONICなど任意のJSONライブラリで読み込む。以下はJacksonを使った例
        final ObjectMapper mapper = new ObjectMapper();
        return mapper.readValue(responseBody, Map.class);
    }

    /**
     * Boundaryに利用するユニークな文字列を返す
     *
     * @return String ユニークな文字列
     */
    private static String getUniqueString() {
        final StringBuilder sb = new StringBuilder();

        final String source = Long.toString(System.currentTimeMillis());
        MessageDigest md = null;
        try {
            md = MessageDigest.getInstance("md5");
            final byte[] hash = md.digest(source.getBytes());

            for (final byte b : hash) {
                sb.append(String.format("%02x", b));
            }
        } catch (final Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        return sb.toString();
    }
}