<?php
/**
 * 顧客一括登録・削除APIの実行サンプル
 */

// レスポンスが文字化けする場合は、ご利用の環境にあわせて文字エンコーディングを変換してください。
const FROM_ENCODE = 'UTF-8';
const TO_ENCODE = 'UTF-8';

// 可変
// 接続先URLに利用する設定
// https://●●●●●/■■■■■/api/v1/custcsv/import
$domain = '●●●●●'; // ログインURLのドメイン部分を指定（xxx.hm-f.jpなど）
$accountUrl = '■■■■■'; // ログインURLのパスの先頭部分

// APIトークン
// 各種設定 > 各種設定メニュー > 基本設定 > APIトークン で発行
$apiToken = "<<APIトークン>>";

// API用のパラメータを指定
// 各パラメータの詳細は仕様書をご確認ください
$importSettings = [
  'import_type' => "★",    // 0:登録, 1:削除, 2:停止, 3:禁止, 5:可能
  'is_overwrite' => "★",   // 1:上書きする, 0:上書きしない
  'overwrite_type' => "★", // 1:元の値を残す, 2:空白で上書きする
  'gid' => "★",            // 配信グループID
  // 'spid' => "★",        // ステップメールプランID
  'is_synchronize' => "★", // 1:配信停止にする, 0:変更しない
  'report_option' => "★",  // 0:送信しない, 1:送信する, 2:エラー時のみ送信する
  'login_id' => "▲▲▲▲▲"     // ご利用のログインID
];

// CSVファイルのパス
$csvFilePath = 'sample.csv';

// マルチパートの境界
$boundary = '-----boundary';

// リクエストのHTTPメッセージ作成
// リクエストボディ
if (!(is_file($csvFilePath) && is_readable($csvFilePath))) {
  die("ファイルが存在しないか、アクセス権限がありません: {$csvFilePath}" . PHP_EOL);
}
$importSettingsJson = json_encode($importSettings);
$csvContent = file_get_contents($csvFilePath);
$requestBody = <<<BODY
--{$boundary}
Content-Disposition: form-data; name="import_settings"

{$importSettingsJson}
--{$boundary}
Content-Disposition: form-data; name="customer_list"; filename="customer_list.csv"

{$csvContent}
--{$boundary}--

BODY;

// リクエストヘッダ
$contentLength = strlen($requestBody);
$requestHeader = <<<HEADER
POST /{$accountUrl}/api/v1/custcsv/import HTTP/1.1
Host: {$domain}
Content-Type: multipart/form-data; boundary="{$boundary}"
Content-Length: {$contentLength}
X-HM-apitoken: {$apiToken}

HEADER;
$requestHeader = preg_replace('/\r?\n/', "\r\n", $requestHeader);

// リクエストの送信
$context = stream_context_create();
stream_context_set_option($context, 'ssl', 'verify_peer', false);
stream_context_set_option($context, 'ssl', 'verify_host', false);

$fp = stream_socket_client('ssl://' . $domain . ':443', $errno, $errstr, 10, STREAM_CLIENT_CONNECT, $context);
if (!$fp) {
  die("接続に失敗しました。errorNo : [ " . $errno . " ]  errorMessage : [ " . $errstr . " ]\n");
}
$requestMessage = $requestHeader . "\r\n" . $requestBody;
fputs($fp, $requestMessage);

// レスポンスの受信
// レスポンスヘッダ
$statusLine = fgets($fp, 4096);
[$httpVer, $statusCode, $message] = explode(' ', $statusLine, 3);

if ($statusCode === '200') {
  print '処理に成功しました' . PHP_EOL;
} else {
  print '処理に失敗しました' . PHP_EOL;
}

// レスポンスボディまで進む
while (!feof($fp)) {
  $temporaryData = fgets($fp, 4096);
  if (preg_match('/^\r?\n/', $temporaryData)) {
    break;
  }
}
// 結果出力
$responseBody = ""; // レスポンスデータ
while (!feof($fp)) {
  $responseBody .= fgets($fp, 4096);
}
fclose($fp);
$jsonData = json_decode($responseBody, true);

print mb_convert_encoding(outputJson($jsonData), TO_ENCODE, FROM_ENCODE) . PHP_EOL;

/**
 * JSONを整形する
 *
 * @param array $jsonData
 * @param int   $depth
 *
 * @return string
 */
function outputJson(?array $jsonData, int $depth = 2) {
  if ($jsonData === null) {
    return "";
  }

  $result = "";
  foreach ($jsonData as $key => $value) {
    if (is_array($value)) {
      $result .= str_repeat(" ", $depth) . $key . " :\n";
      $depth += 2;
      $result .= outputJson($value, $depth);
      $depth -= 2;
    } else {
      $result .= str_repeat(" ", $depth) . $key . " : " . $value . "\n";
    }
  }
  return $result;
}
