#
# APIのサンプル
#
# 下記バージョンで動作確認しています。
# ＜動作確認バージョン＞
# perl v5.26
#
# 下記モジュールが必要です。
# ＜必要なモジュール＞
# perl-libwww-perl
# (example: yum install perl-libwww-perl)
# perl-JSON
# (example: yum install perl-JSON)
# LWP::Protocol::https
# (example: cpan install LWP::Protocol::https)
#
use HTTP::Request::Common;
use LWP::UserAgent;
use JSON;


# 可変
# 接続先URLに利用する設定
# https://●●●●●/■■■■■/api/v1/custcsv/get
my $domain = "●●●●●";      # URLのドメイン部分を指定（xxx.hm-f.jpなど）
my $accountUrl = "■■■■■";  # アカウント文字列
# APIトークン
# 各種設定 > 各種設定メニュー > 基本設定 > APIトークン で発行
my $apiToken = "<<APIトークン>>";

# API用のパラメータを指定
# 各パラメータの詳細は仕様書をご確認いただき、"<<...>>"を書き換えてください。
# (gid, spid を同時に指定することはできません)
my $requestJson = <<REQ_JSON;
{
  "import_type" : "<<0:登録, 1:削除, 2:停止, 3:禁止, 5:可能>>",
  "is_overwrite" : "<<1:上書きする, 0:上書きしない>>",
  "overwrite_type" : "<<1:元の値を残す, 2:空白で上書きする>>",
  "gid" : "<<配信グループID>>",
  "spid" : "<<ステップメールプランID>>,
  "is_synchronize" : "<<1:配信停止にする, 0:変更しない>>",
  "report_option" : "<<0:送信しない, 1:送信する, 2:エラー時のみ送信する>>",
  "login_id" : "<<ご利用のログインID>>"
}
REQ_JSON

# CSVファイルのパス
my $uploadCsvFilePath = "<<CVSファイルのパス>>";
# マルチパートの境界
my $boundary = '-----boundary';

my $requestData = <<REQ_DATA;
--$boundary
Content-Disposition: form-data; name="import_settings"

$requestJson
--$boundary
Content-Disposition: form-data; name="customer_list"; filename="customer_list.csv"

REQ_DATA

open (CSVFILE, "<", $uploadCsvFilePath) or die("処理に失敗しました。[message=$!]");
while(my $line = <CSVFILE>){
  $requestData .= "$line";
}
close(CSVFILE);
$requestData .= "\n--$boundary--";

my $url = "https://${domain}/${accountUrl}/api/v1/custcsv/import";
my $userAgent = LWP::UserAgent->new();
# SSL証明書エラー対策
# $userAgent->ssl_opts(verify_hostname => 0);
my $header = HTTP::Headers->new;
$header->push_header('Content-Type' => 'multipart/form-data; boundary="' . $boundary . '"');
$header->push_header('X-HM-apitoken' => $apiToken);
$header->push_header('X-Content-Type-Options' => 'nosniff');
$header->push_header('Connection' => 'Close');

my $reqest = HTTP::Request->new('POST', $url, $header, $requestData);
my $response = $userAgent->request($reqest);

if (!$response->is_success || $response->status_line ne '200 OK') {
  if ($response->content_type eq 'application/json') {
    my $data = decode_json($response->content);
    printf "処理に失敗しました。[line=%s]\n", __LINE__;
    printf "  status : %s\n  code : %s\n  url : %s\n  query : %s\n  errors : %s\n  version : %s\n  accessTime : %s\n"
      , $data->{'status'}
      , $data->{'code'}
      , $data->{'url'}
      , encode_json($data->{'query'})
      , encode_json($data->{'errors'})
      , $data->{'version'}
      , $data->{'accessTime'}
  } else {
    printf "処理に失敗しました。[line=%s][content=%s]\n", __LINE__, $response->content
  }
  die;
}

print "処理に成功しました。\n";
exit;
